# Generated by roxygen2: do not edit by hand

export("%>%")
export(DEBUG)
export(ERROR)
export(FATAL)
export(INFO)
export(TRACE)
export(WARN)
export(check)
export(check_fast)
export(check_faster)
export(check_fastest)
export(check_slow)
export(check_slower)
export(check_slowest)
export(createLocationID)
export(createLocationMask)
export(dateRange)
export(dateSequence)
export(getAPIKey)
export(html_getLinkNames)
export(html_getLinkUrls)
export(html_getLinks)
export(html_getTable)
export(html_getTables)
export(initializeLogging)
export(lintFunctionArgs_dir)
export(lintFunctionArgs_file)
export(loadDataFile)
export(logger.debug)
export(logger.error)
export(logger.fatal)
export(logger.info)
export(logger.isInitialized)
export(logger.setLevel)
export(logger.setup)
export(logger.trace)
export(logger.warn)
export(manageCache)
export(parseDatetime)
export(setAPIKey)
export(setIfNull)
export(showAPIKeys)
export(stopIfNull)
export(stopOnError)
export(timeRange)
export(timeStamp)
export(timezoneLintRules)
export(validateLonLat)
export(validateLonsLats)
importFrom(logger,appender_console)
importFrom(logger,appender_file)
importFrom(logger,appender_tee)
importFrom(logger,appender_void)
importFrom(logger,formatter_sprintf)
importFrom(logger,layout_simple)
importFrom(logger,log_appender)
importFrom(logger,log_debug)
importFrom(logger,log_error)
importFrom(logger,log_fatal)
importFrom(logger,log_formatter)
importFrom(logger,log_info)
importFrom(logger,log_layout)
importFrom(logger,log_threshold)
importFrom(logger,log_trace)
importFrom(logger,log_warn)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
importFrom(utils,str)
