% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-location.R
\name{validateLonLat}
\alias{validateLonLat}
\title{Validate longitude and latitude values}
\usage{
validateLonLat(longitude = NULL, latitude = NULL)
}
\arguments{
\item{longitude}{Single longitude in decimal degrees east.}

\item{latitude}{Single latitude in decimal degrees north.}
}
\value{
Invisibly returns \code{TRUE} if validation succeeds.
}
\description{
Validate a single longitude/latitude pair to ensure both values are
numeric scalars and fall within valid geographic bounds.
}
\details{
Longitudes must fall between -180 and 180 degrees and latitudes must
fall between -90 and 90 degrees. If validation fails, an error is
generated.
}
\examples{
validateLonLat(-122.5, 47.5)

\dontrun{
validateLonLat(-200, 47.5)
validateLonLat(-122.5, NA)
}

}
