% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devtools_check.R
\name{packageCheck}
\alias{packageCheck}
\alias{check_slow}
\alias{check}
\alias{check_fast}
\alias{check_faster}
\alias{check_fastest}
\alias{check_slower}
\alias{check_slowest}
\title{Run package checks}
\usage{
check(pkg = ".")

check_fast(pkg = ".")

check_faster(pkg = ".")

check_fastest(pkg = ".")

check_slow(pkg = ".")

check_slower(pkg = ".")

check_slowest(pkg = ".")
}
\arguments{
\item{pkg}{Package location passed to \code{\link[devtools:check]{devtools::check()}}.}
}
\value{
Invisibly returns the result from \code{\link[devtools:check]{devtools::check()}}.
}
\description{
Convenience wrappers around \code{\link[devtools:check]{devtools::check()}} for package checking at
different levels of thoroughness.
}
\details{
These functions make it easy to run quick checks during active development
and more thorough checks before merging or releasing package changes.

The functions are ordered from most thorough to fastest:

\describe{
\item{\code{check_slowest()}}{
Builds the manual, runs \code{donttest} and \code{dontrun} examples, and
uses \code{--use-gct}.
}
\item{\code{check_slower()}}{
Builds the manual and runs \code{donttest} and \code{dontrun} examples.
}
\item{\code{check_slow()}}{
Builds the manual and runs \code{donttest} examples.
}
\item{\code{check()}}{
Standard development check without building the manual or running
\code{donttest} examples.
}
\item{\code{check_fast()}}{
Skips vignette building and ignores vignettes during checking.
}
\item{\code{check_faster()}}{
Skips vignette building, ignores vignettes, and skips examples.
}
\item{\code{check_fastest()}}{
Skips vignette building, ignores vignettes, skips examples, and skips
tests.
}
}
}
\seealso{
\code{\link[devtools:check]{devtools::check()}}
}
