% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-logging.R
\name{logger.setLevel}
\alias{logger.setLevel}
\title{Set console log level}
\usage{
logger.setLevel(level)
}
\arguments{
\item{level}{Logging threshold level.}
}
\value{
No return value. Called for side effects.
}
\description{
Set the minimum log level displayed in the console.
}
\details{
By default, only \code{FATAL} messages are displayed in the console. This
function allows users to display additional log messages interactively.

Available log levels are:

\preformatted{
TRACE
DEBUG
INFO
WARN
ERROR
FATAL
}
}
\note{
All functionality is implemented with the excellent \pkg{logger} package.
}
\examples{
\dontrun{
# Enable console logging
logger.setup()

# Show DEBUG and higher messages in the console
logger.setLevel(DEBUG)
}

}
\seealso{
\code{\link[=logger.setup]{logger.setup()}}
}
