% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-tidyeval-data.R
\name{tidyeval-data}
\alias{tidyeval-data}
\alias{.data}
\title{Data pronoun for tidy evaluation}
\description{
See \link[rlang:dot-data]{rlang::.data} for details.
}
\keyword{internal}
